"""
Django command for converting a section into a course.
"""
import sys

from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand
from opaque_keys import InvalidKeyError
from opaque_keys.edx.keys import CourseKey
from opaque_keys.edx.locator import BlockUsageLocator

from section_to_course.compat import not_found_exception
from section_to_course.utils import paste_from_template

User = get_user_model()


class Command(BaseCommand):
    """
    Management command to convert a section into a course.
    """

    help = 'Converts a section into a course'

    def add_arguments(self, parser):
        parser.add_argument('source_section_id', type=str)
        parser.add_argument('destination_course_id', type=str)
        parser.add_argument('username', type=str)

    def handle(self, *args, **options):
        try:
            user = User.objects.get(username=options['username'])
        except User.DoesNotExist:
            self.stderr.write(self.style.ERROR(f'User "{options["username"]}" does not exist.'))
            sys.exit(1)
        try:
            destination_course_key = CourseKey.from_string(options['destination_course_id'])
        except InvalidKeyError:
            self.stderr.write(self.style.ERROR(f'"{options["destination_course_id"]}" is not a valid course key.'))
            sys.exit(2)
        try:
            source_block_usage_key = BlockUsageLocator.from_string(options['source_section_id'])
        except InvalidKeyError:
            self.stderr.write(self.style.ERROR(f'"{options["source_section_id"]}" is not a valid block usage key.'))
            sys.exit(3)
        try:
            paste_from_template(
                destination_course_key=destination_course_key,
                source_block_usage_key=source_block_usage_key,
                user=user,
            )
        except not_found_exception() as err:
            self.stderr.write(self.style.ERROR(str(err)))
            sys.exit(4)
        self.stdout.write(self.style.SUCCESS('Section copied successfully.'))



# Generated by Django 3.2.23 on 2023-11-15 10:05

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('section_to_course', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sectiontocourselink',
            name='last_refresh',
            field=models.DateTimeField(blank=True, default=django.utils.timezone.now, null=True),
        ),
    ]



"""
Database models for section_to_course.
"""
# from django.db import models
from django.db import models
from django.utils import timezone
from model_utils.models import TimeStampedModel
from opaque_keys.edx.django.models import CourseKeyField, UsageKeyField


class SectionToCourseLink(TimeStampedModel):
    """
    A model that links a section to a course.

    .. no_pii:
    """

    source_course_id = CourseKeyField(max_length=255, db_index=True, null=False, blank=False)
    destination_course_id = CourseKeyField(max_length=255, db_index=True, null=False, blank=False)
    source_section_id = UsageKeyField(max_length=255, db_index=True, null=False, blank=False)
    destination_section_id = UsageKeyField(max_length=255, db_index=True, null=False, blank=False)
    last_refresh = models.DateTimeField(null=True, blank=True, default=timezone.now)

    class Meta:
        """Meta settings for SectionToCourseLink model."""

        unique_together = ('source_course_id', 'destination_course_id', 'source_section_id')

    def __str__(self):
        """
        Get a string representation of this model instance.
        """
        return f'<SectionToCourseLink #{self.id}, {str(self.source_course_id).split(":")[-1]} to ' \
               f'{str(self.destination_course_id).split(":")[-1]} for {str(self.source_section_id).split("@")[-1]}>'



"""
section_to_course Django application initialization.
"""

from django.apps import AppConfig
from edx_django_utils.plugins.constants import PluginSettings, PluginURLs


class SectionToCourseConfig(AppConfig):
    """
    Configuration for the section_to_course Django application.
    """

    name = 'section_to_course'

    plugin_app = {
        # Configuration setting for Plugin URLs for this app.
        PluginURLs.CONFIG: {
            'cms.djangoapp': {
                PluginURLs.NAMESPACE: 'section_to_course',
                # The application namespace to provide to django's urls.include.
                # Optional; Defaults to None.
                PluginURLs.APP_NAME: 'section_to_course',

                # The regex to provide to django's urls.url.
                # Optional; Defaults to r''.
                PluginURLs.REGEX: r'^section-to-course/api/',

                # The python path (relative to this app) to the URLs module to be plugged into the project.
                # Optional; Defaults to 'urls'.
                PluginURLs.RELATIVE_PATH: 'api.urls',
            }
        },
        PluginSettings.CONFIG: {
            'cms.djangoapp': {
                'common': {
                    PluginSettings.RELATIVE_PATH: 'settings.common',
                },
            }
        },
    }
